mutation withCreateBoleto_Mutation(
  $input: CreateTransferInBoletoInput!
) {
  createTransferInBoleto(input: $input) {
    boleto {
      id
      dueDate
      barcode
      readableBarcode
      amount
    }
  }
}
